package com.elitesland.demo.repo;

import com.elitesland.demo.entity.QCompanyDO;
import com.elitesland.demo.entity.QEmployeeDO;
import com.elitesland.demo.param.EmpCompanyQueryParam;
import com.elitesland.demo.vo.EmpCompanyVO;
import com.querydsl.core.types.ExpressionUtils;
import com.querydsl.core.types.Predicate;
import com.querydsl.core.types.Projections;
import com.querydsl.jpa.impl.JPAQuery;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 功能说明
 * </p>
 *
 * @author Mir
 * @date 2020/6/29
 */
@Component
@RequiredArgsConstructor
public class EmployeeRepoProc {

    private final JPAQueryFactory jpaQueryFactory;

    public JPAQuery<EmpCompanyVO> select() {
        val employee = QEmployeeDO.employeeDO;
        val company = QCompanyDO.companyDO;

        return jpaQueryFactory.select(
                Projections.bean(
                        EmpCompanyVO.class,
                        employee.name.as("employeeName"),
                        company.companyName.as("companyName"),
                        company.businessLicense
                )
        )
                .from(employee)
                .leftJoin(company).on(employee.companyId.eq(company.id));
    }

    public Predicate where(EmpCompanyQueryParam param){
        val employee = QEmployeeDO.employeeDO;
        val company = QCompanyDO.companyDO;

        Predicate predicate = employee.isNotNull().or(employee.isNull());
        predicate = StringUtils.isBlank(param.getEmployeeName()) ? predicate : ExpressionUtils.and(predicate, employee.name.like("%" + param.getEmployeeName() + "%"));
        predicate = StringUtils.isBlank(param.getCompanyName()) ? predicate : ExpressionUtils.and(predicate, company.companyName.like("%" + param.getCompanyName() + "%"));
        predicate = StringUtils.isBlank(param.getBusinessLicense()) ? predicate : ExpressionUtils.and(predicate, company.businessLicense.like("%" + param.getBusinessLicense() + "%"));

        return predicate;

    }
}
